import * as Popper from '@webbo-ui/popper';
import { OptionProps } from './option.js';
import { AutocompleteVariantProps, AutocompleteClassNames } from '@webbo-ui/theme';
import React__default from 'react';

type Reason = 'select' | 'clear' | 'remove';
interface RenderInputProps<Value> {
    selected: {
        label: string;
        onDelete: () => void;
    }[] | null;
    inputRef: React__default.RefObject<HTMLInputElement>;
    popperReferenceRef: (instance: HTMLDivElement | null) => void;
    disabled: boolean;
    isOpen: boolean;
    multiple: boolean;
    value: Value;
    inputValue: string;
    showClearButton: boolean;
    onBlur: () => void;
    onOpen: () => void;
    clearButtonProps: {
        onClear: (e: React__default.PointerEvent) => void;
        onPointerDown: (e: React__default.PointerEvent) => void;
        tabIndex: number;
        'aria-label': string;
    };
    onChange: React__default.ChangeEventHandler<HTMLInputElement>;
    onKeyDown: React__default.KeyboardEventHandler<HTMLInputElement>;
    ariaProps: {
        'aria-expanded': boolean;
        'aria-controls': string;
        'aria-haspopup': 'listbox';
        'aria-autocomplete': 'list';
        'aria-activedescendant': string | undefined;
        role: 'combobox';
    };
}
type AutocompleteProps<Value, Multiple, DisableClearable> = (AutocompleteVariantProps & Omit<React__default.HTMLAttributes<HTMLUListElement>, 'defaultValue' | 'children'> & {
    disabled?: boolean;
    classNames?: AutocompleteClassNames;
    offset?: Popper.FloatingProps['mainOffset'];
    options: Value[];
    isOpen?: boolean;
    onOpenChange?: (open: boolean) => void;
    defaultOpen?: boolean;
    getOptionDisabled?: (option: Value) => boolean;
    disableCloseOnSelect?: boolean;
    getOptionLabel?: (option: Value) => string;
    getOptionKey?: (options: Value) => string;
    noOptionsText?: string;
    loading?: boolean;
    loadingText?: string;
    groupBy?: (option: Value) => string;
    children?: (props: {
        groupedOptions: Record<string, OptionProps<Value>[]> | null;
        options: OptionProps<Value>[] | null;
    }) => React__default.ReactNode;
    inputValue?: string;
    onInputChange?: (val: string) => void;
    filterOptions?: (options: Value[], inputValue: string) => Value[];
}) & (Multiple extends true ? {
    multiple: Multiple;
    defaultValue?: Value[];
    value?: Value[];
    onChange?: (event: {
        target: {
            value: Value[];
        };
    }, reason: Reason, value: Value[]) => void;
    disableClearable?: DisableClearable;
    renderInput: (props: RenderInputProps<Value[]>) => React__default.ReactNode;
} : DisableClearable extends true ? {
    multiple?: Multiple;
    defaultValue?: Value;
    value?: Value;
    onChange?: (event: {
        target: {
            value: Value;
        };
    }, reason: Reason, value: Value) => void;
    disableClearable: DisableClearable;
    renderInput: (props: RenderInputProps<Value>) => React__default.ReactNode;
} : {
    multiple?: Multiple;
    defaultValue?: Value;
    value?: Value | null;
    onChange?: (event: {
        target: {
            value: Value | null;
        };
    }, reason: Reason, value: Value | null) => void;
    disableClearable?: DisableClearable;
    renderInput: (props: RenderInputProps<Value | null>) => React__default.ReactNode;
});
declare const Autocomplete: <Value extends object, Multiple extends boolean = false, DisableClearable extends boolean = false>(props: AutocompleteProps<Value, Multiple, DisableClearable> & React__default.RefAttributes<HTMLUListElement>) => React__default.ReactNode;

export { Autocomplete, type AutocompleteProps, type Reason, type RenderInputProps };
