"use client";

// src/option.tsx
import { mergeRefs } from "@webbo-ui/react-utils";
import { usePointerEvents } from "@webbo-ui/use-pointer-events";
import React from "react";
import { jsx } from "react/jsx-runtime";
var Option = React.forwardRef((props, ref) => {
  const {
    label,
    onSelect,
    onHover,
    state: { disabled, focused, selected },
    children,
    onPointerDown,
    onPointerUp,
    onPointerEnter,
    onPointerLeave,
    ...restProps
  } = props;
  const innerRef = React.useRef(null);
  const isHovered = React.useRef(false);
  const pointerEvents = usePointerEvents({
    onPointerDown: (e) => {
      onPointerDown?.(e);
      e.preventDefault();
    },
    onPointerUp,
    onPress: () => {
      if (disabled)
        return;
      onSelect();
    }
  });
  React.useEffect(() => {
    if (selected)
      innerRef.current?.scrollIntoView({
        behavior: "instant",
        block: "center"
      });
  }, [selected]);
  React.useEffect(() => {
    if (focused && !isHovered.current)
      innerRef.current?.scrollIntoView({
        behavior: "instant",
        block: "nearest"
      });
  }, [focused]);
  return /* @__PURE__ */ jsx(
    "li",
    {
      ref: mergeRefs(ref, innerRef),
      onPointerEnter: (e) => {
        onPointerEnter?.(e);
        isHovered.current = true;
        onHover();
      },
      onPointerLeave: (e) => {
        onPointerLeave?.(e);
        isHovered.current = false;
      },
      ...pointerEvents,
      ...restProps,
      children: children ?? label
    }
  );
});
Option.displayName = "webbo-ui.Option";

export {
  Option
};
